@@ -65,6 +65,9 @@ def d_filter_wrapper(wd, **kwargs):
         logging.info("Will filter the genome list")
         bdb = drep.d_cluster.utils.load_genomes(kwargs['genomes'])
 
+    # Quick sanity check
+    sanity_check(bdb)
+
     # Calculate the length and N50 of all genomes
     logging.info("Calculating genome info of genomes")
     Gdb = calc_genome_info(bdb['location'].tolist())
@@ -93,6 +96,40 @@ def d_filter_wrapper(wd, **kwargs):
     if not kwargs.get('ignoreGenomeQuality', False):
         workDirectory.store_db(Gdb, 'genomeInfo')
 
+def sanity_check(bdb, **kwargs):
+    """
+    Make sure there are no duplicate names or anything
+
+    Args:
+        bdb:
+
+    Returns:
+
+    """
+    # Check for duplicated genome names
+    dupped = [g for g, c in bdb['genome'].value_counts().to_dict().items() if c > 1]
+    if len(dupped) > 0:
+        logging.error("You have duplicate genome basenames! This doesn't work for dRep; each basename needs to be unique. Here are you problem genomes:")
+        logging.error(bdb[bdb['genome'].isin(dupped)].sort_values('genome'))
+        raise Exception
+
+    # Report number of input genomes
+    logging.info(f"{len(bdb):,} genomes were input to dRep")
+
+    # Make a suggestion about greedy?
+    pc =  kwargs.get('primary_chunksize', None)
+    if pc is not None:
+        mc = kwargs.get('multiround_primary_clustering', False)
+        if (len(bdb) > pc) & (mc == False):
+            logging.info(f"Hey! You have {len(bdb)} genomes and arent using greedy algorithms. See https://drep.readthedocs.io/en/latest/choosing_parameters.html#using-greedy-algorithms for info on how to make your run faster")
+
+    # If you mc, try and tell them to turn on tc
+    mc = kwargs.get('multiround_primary_clustering', False)
+    tc = kwargs.get('run_tertiary_clustering', False)
+    if mc and not tc:
+        logging.info(f"Hey! You're running multiround_primary_clustering but not run_tertiary_clustering! You should add --run_tertiary_clustering when running with multiround_primary_clustering to avoid weird placement; see https://drep.readthedocs.io/en/latest/choosing_parameters.html#using-greedy-algorithms for more info")
+
+
 def _get_run_genomeInfo(workDirectory, bdb, **kwargs):
     '''
     Through kwargs and the wd, get genomeInfo
@@ -116,7 +153,7 @@ def _get_run_genomeInfo(workDirectory, bdb, **kwargs):
             Tdb = _validate_genomeInfo(Idb, bdb)
             Gdb = _add_lengthN50(Tdb, bdb)
         except:
-            Idb = pd.read_table(kwargs.get('genomeInfo'))
+            Idb = pd.read_csv(kwargs.get('genomeInfo'), sep='\t')
             Tdb = _validate_genomeInfo(Idb, bdb)
             Gdb = _add_lengthN50(Tdb, bdb)
 
@@ -320,7 +357,7 @@ def validate_chdb(Chdb, bdb):
             logging.error("{0} is not in checkM db".format(genome))
             quit = True
     if quit:
-        logging.error("New checkM db needs to be made")
+        logging.error("New checkM db needs to be made\nSee https://drep.readthedocs.io/en/latest/advanced_use.html#troubleshooting-checkm for help troubleshooting")
         sys.exit()
     return
 
@@ -460,7 +497,7 @@ def run_prodigal(genome_list, out_dir, **kwargs):
         if os.path.exists(fna) and os.path.exists(faa):
             pass
         else:
-            cmds.append(['prodigal','-i',genome,'-d',fna,'-a',faa,'-m','-p','meta'])
+            cmds.append(['prodigal','-i',genome,'-d',fna,'-a',faa,'-m','-p','single'])
 
     # Run commands
     if len(cmds) > 0:
@@ -475,11 +512,11 @@ def run_prodigal(genome_list, out_dir, **kwargs):
     else:
         logging.info("Past prodigal runs found- will not re-run")
 
-def run_checkM(genome_folder,checkm_outf,**kwargs):
+def run_checkM(genome_folder_whole, checkm_outf_whole, **kwargs):
     '''
     Run checkM
 
-    WARNING- this will result in wrong genome lenth and genome N50 estimate, due to
+    WARNING- this will result in wrong genome length and genome N50 estimate, due to
     it being run on prodigal output
 
     Args:
@@ -493,21 +530,11 @@ def run_checkM(genome_folder,checkm_outf,**kwargs):
         wd: if you want to log commands, you also need the wd
         set_recursion: if not 0, set the python recursion
     '''
-    # Find checkm exe
-    loc, works = drep.d_bonus.find_program('checkm')
-    if loc == None:
-        logging.error('Cannot locate the program {0}- make sure its in the system path'\
-            .format('checkm'))
-        sys.exit()
-    if works == False:
-        logging.error('Program {0} is not working!! Im going to crash now'\
-            .format('checkm'))
-        sys.exit()
-    check_exe = loc
-
     # Get set up
+    check_exe = _checkm_get_exe()
     t = str(kwargs.get('processors','6'))
-    checkm_method = kwargs.get('checkM_method','lineage_wf')
+    checkm_method = kwargs.get('checkM_method', 'lineage_wf')
+    checkm_group_size = kwargs.get('checkm_group_size', 1000)
 
     # Set recursion
     R = kwargs.get('set_recursion', '0')
@@ -515,48 +542,98 @@ def run_checkM(genome_folder,checkm_outf,**kwargs):
         logging.warning('Setting Maximum Recursion depth to {0}'.format(R))
         sys.setrecursionlimit(int(R))
 
-    # Run checkM initial
-    if checkm_method == 'taxonomy_wf':
-         cmd = [check_exe,checkm_method,'domain','Bacteria',genome_folder,checkm_outf,'-f',\
-            checkm_outf + '/results.tsv','--tab_table','-t',str(t),'-g','-x','faa']
-    else:
-         cmd = [check_exe,checkm_method,genome_folder,checkm_outf,'-f',\
-            checkm_outf + '/results.tsv','--tab_table','-t',str(t),'--pplacer_threads',\
-            str(t),'-g','-x','faa']
+    # Establish groups
+    dbs = []
+    for genome_folder, checkm_outf in _iterate_checkm_groups(genome_folder_whole, checkm_outf_whole, checkm_group_size):
 
-    logging.debug("Running CheckM with command: {0}".format(cmd))
+        # Run checkM initial
+        if checkm_method == 'taxonomy_wf':
+             cmd = [check_exe,checkm_method,'domain','Bacteria',genome_folder,checkm_outf,'-f',\
+                checkm_outf + '/results.tsv','--tab_table','-t',str(t),'-g','-x','faa']
+        else:
+             cmd = [check_exe,checkm_method,genome_folder,checkm_outf,'-f',\
+                checkm_outf + '/results.tsv','--tab_table','-t',str(t),'--pplacer_threads',\
+                str(t),'-g','-x','faa']
 
-    if ('wd' in kwargs) & (kwargs.get('debug', False) == True):
-        logdir = kwargs.get('wd').get_dir('cmd_logs')
-    else:
-        logdir = False
-    drep.run_cmd(cmd, shell=False, logdir=logdir)
+        logging.debug("Running CheckM with command: {0}".format(' '.join(cmd)))
 
-    # Run checkM again for the better table
-    if checkm_method == 'taxonomy_wf':
-        lineage = checkm_outf + 'Bacteria.ms'
-    else:
-        lineage = checkm_outf + 'lineage.ms'
-    desired_file = checkm_outf + 'Chdb.tsv'
-    cmd = [check_exe,'qa', lineage, checkm_outf, '-f', desired_file, '-t',\
-            str(t), '--tab_table','-o', '2']
-    logging.debug("Running CheckM with command: {0}".format(cmd))
+        if ('wd' in kwargs) & (kwargs.get('debug', False) == True):
+            logdir = kwargs.get('wd').get_dir('cmd_logs')
+        else:
+            logdir = False
+        drep.run_cmd(cmd, shell=False, logdir=logdir)
 
-    if ('wd' in kwargs) & (kwargs.get('debug', False) == True):
-        logdir = kwargs.get('wd').get_dir('cmd_logs')
-    else:
-        logdir = False
-    drep.run_cmd(cmd, shell=False, logdir=logdir)
+        # Run checkM again for the better table
+        if checkm_method == 'taxonomy_wf':
+            lineage = checkm_outf + 'Bacteria.ms'
+        else:
+            lineage = checkm_outf + 'lineage.ms'
+        desired_file = checkm_outf + 'Chdb.tsv'
+        cmd = [check_exe,'qa', lineage, checkm_outf, '-f', desired_file, '-t',\
+                str(t), '--tab_table','-o', '2']
+        logging.debug("Running CheckM with command: {0}".format(' '.join(cmd)))
 
-    # Load table
-    try:
-        chdb = pd.read_table(desired_file,sep='\t')
-    except:
-        logging.error("!!! checkM failed !!!\nYou can run again with the --debug option to see what went wrong (command logs will be created in the log folder)")
+        if ('wd' in kwargs) & (kwargs.get('debug', False) == True):
+            logdir = kwargs.get('wd').get_dir('cmd_logs')
+        else:
+            logdir = False
+        drep.run_cmd(cmd, shell=False, logdir=logdir)
+
+        # Load table
+        try:
+            chdb = pd.read_table(desired_file,sep='\t')
+            chdb['Bin Id'] = chdb['Bin Id'].astype(str)
+        except:
+            logging.error("!!! checkM failed !!!\nSee https://drep.readthedocs.io/en/latest/advanced_use.html#troubleshooting-checkm for help troubleshooting")
+            sys.exit()
+
+        # Return table
+        dbs.append(chdb)
+
+    return pd.concat(dbs).reset_index(drop=True)
+
+def _checkm_get_exe():
+    loc, works = drep.d_bonus.find_program('checkm')
+    if loc == None:
+        logging.error('Cannot locate the program {0}- make sure its in the system path' \
+                      .format('checkm'))
         sys.exit()
+    if works == False:
+        logging.error('Program {0} is not working!! Im going to crash now' \
+                      .format('checkm'))
+        sys.exit()
+    return loc
 
-    # Return table
-    return chdb
+def _iterate_checkm_groups(genome_folder_whole, checkm_outf_whole, checkm_group_size):
+    """
+    Break up the checkM input into groups of checkm_group_size
+    """
+    # Get the number of prodigal files you have
+    total_genes = glob.glob(os.path.join(genome_folder_whole, '*faa'))
+    chunks = [total_genes[x:x + checkm_group_size] for x in range(0, len(total_genes), checkm_group_size)]
+
+    if len(chunks) == 1:
+        yield genome_folder_whole, checkm_outf_whole
+
+    else:
+        logging.info(f"Running checkM in {len(chunks)} chunks")
+
+        if not os.path.isdir(checkm_outf_whole):
+            os.mkdir(checkm_outf_whole)
+
+        for i, chunk in enumerate(chunks):
+            logging.info(f"Running checkM chunk {i}")
+
+            new_checkm_out = os.path.join(checkm_outf_whole, f"genome_chunk_{i}_out/")
+            new_checkm_prods = os.path.join(checkm_outf_whole, f"genome_chunk_{i}_genes")
+
+            os.mkdir(new_checkm_out)
+            os.mkdir(new_checkm_prods)
+
+            for faa in chunk:
+                os.symlink(faa, os.path.join(new_checkm_prods, os.path.basename(faa)))
+
+            yield new_checkm_prods, new_checkm_out
 
 def calc_fasta_length(fasta_loc):
     '''
